package org.rexxla.bsf;

import java.io.*;

/** Invokes given Rexx or ooRexx program and supplies the commandline arguments.
 *
 *  Compiled such that it is able to use IBM's BSF (needs Java &gt;= 1.1) or
 *  the newer Apache BSF (needs Java &gt;= 1.3); to recompile you therfore need
 *  both packages; if in doubt drop the support of the (older) IBM BSF.
 *
 * <pre>------------------------ Apache Version 2.0 license -------------------------
 *    Copyright (C) 2006 Rony G. Flatscher
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        <a href="http://www.apache.org/licenses/LICENSE-2.0">http://www.apache.org/licenses/LICENSE-2.0</a>
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 * ----------------------------------------------------------------------------- </pre>
 *
 * @author   Rony G. Flatscher
 * @version  1.1, 2006-01-02
 */

 /*  2006-01-07, ---rgf, removed ability to look for com.ibm.bsf, if org.apache.bsf is not
                         found (to reduce confusion on error reports)
 */
public class RexxDispatcher {
        /** Version string indicating version of this class (majorVersion*100+minorVersion
         *  concatenated with a dot and the sorted date of last change.
         */
        static public String version = "101.20060107";

        /** Key for storing the received args-String array in the BSF registry for Rexx to
         *  get at it.
         */
        static private String keyName="allCommandLineArguments";

        private RexxDispatcher() {};   // do not allow to create an instance of this class

	/**
	 * Dispatches the Rexx/ooRexx program in args[0] and supplies it with the argument,
         * assembled by concatenating all remaining arguments with a blank. In addition the
         * String array <code>args[]</code> is saved in the BSF registry under the name
         * &quot;allCommandLineArguments&quot;.
	 *
	 * @param args arguments
	 *
	 */
	public static void main(String[] args)
        {
            if (args.length == 0)
            {
                    System.err.println("usag:\n\tjava RexxDispatcher rexx_file_name [arguments]");
                    System.exit(1);
            }

            String [] args4rexx    = null;
            String    argline      = null;
            java.util.Vector vArgs = null;

            Object scriptString    = null;
            boolean bApache = false;
            org.apache.bsf.BSFManager ap_mgr  = null;   // Apache's bsf, Java >= 1.3
            com.ibm.bsf.BSFManager   ibm_mgr  = null;   // IBM's bsf, Java >= 1.1

            org.apache.bsf.BSFEngine  ap_rexx = null;   // Apache's bsf, Java >= 1.3
            com.ibm.bsf.BSFEngine    ibm_rexx = null;   // IBM's bsf, Java >= 1.1

            try     // access to Apache's BSF there ?
            {
                ap_mgr  = new org.apache.bsf.BSFManager ();
                ap_mgr.declareBean(keyName, args, args.getClass());    // save args[] in registry
                ap_rexx = ap_mgr.loadScriptingEngine("rexx");
                bApache = true;         // o.k., use Apache's BSF
            }
            catch (Exception e)     // hmm, do we have access to IBM's BSF, and if so, use it!
            {
                e.printStackTrace();
                System.exit(1);
            }



/*
                try
                {
                    ibm_mgr  = new com.ibm.bsf.BSFManager ();
                    ibm_mgr.declareBean(keyName, args, args.getClass());    // save args[] in registry
                    ibm_rexx = ibm_mgr.loadScriptingEngine("rexx");
                    bApache  = false;   // indicate that IBM's BSF should be used
                }
                catch (Exception e1)
                {
                    System.err.println("Could not find org.apache.bsf.* classes:\n");
                    e.printStackTrace();
                    System.err.println("\n... nor the com.ibm.bsf.* classes:\n");
                    e1.printStackTrace();
                    System.err.println("... aborting.");
                    System.exit(1);
                }
            }

                // if additional arguments, pass them as a blank delimited string to Rexx
            if (args.length>1)
            {
                vArgs=new java.util.Vector();
                argline="";
                boolean bEnquote=false;

                for (int i=1; i<args.length; i++)
                {
                    argline=argline + (argline.length()>0 ? " " : "");  // append a blank, if not first argument

                    // if an argument has a space re-enquote it!
                    bEnquote=false;
                    for (int k=0; k<args[i].length() && bEnquote==false; k++)
                    {
                        bEnquote=(args[i].charAt(k)==' ');
                    }
                    argline=argline + (bEnquote==true ? "\"" : "") + args[i] + (bEnquote==true ? "\"" : "");
                }
                vArgs.addElement(argline);      // add concatenated String
            }

*/
            try
            {
                // org.apache.bsf ...
                if (bApache==true)
                {
                        // execute Rexx script
                    Object obj =ap_rexx.apply( args[0],
                                            0,
                                            0,
                                            (Object) org.apache.bsf.util.IOUtils.getStringFromReader(new FileReader(args[0])),
                                            null,
                                            vArgs
                                          );
                }

                else          // com.ibm.bsf ...
                {
                        // execute Rexx script
                    Object obj =ibm_rexx.apply( args[0],
                                            0,
                                            0,
                                            (Object) com.ibm.bsf.util.IOUtils.getStringFromReader(new FileReader(args[0])),
                                            null,
                                            vArgs
                                          );
                }
            }
            catch (Exception e)
            {
                e.printStackTrace();
                System.exit(1);
            }

            System.exit(0);
	}
}
